"""Django connector for Elastic App Search."""

# pylint:disable=invalid-name
# Django requires `default_app_config`
default_app_config = 'django_elastic_appsearch.apps.DjangoAppSearchConfig'
__version__ = '0.5.5'
